/*
 * deviced
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <stdbool.h>
#include <Ecore.h>
#include <hw/thermal.h>

#include "apps/apps.h"
#include "core/devices.h"
#include "core/log.h"
#include "core/device-notifier.h"

static struct thermal_device *thermal_dev;

static int noti;

static void thermal_remove_noti(void)
{
	if (noti) {
		remove_notification("TempCooldownNotiOff", noti);
		noti = 0;
	}
}

static void thermal_add_noti(void)
{
	if (!noti)
		noti = add_notification("TempCooldownNotiOn");
}

static void thermal_add_popup(void)
{
	launch_system_app(APP_OVERHEAT, 2,
			APP_KEY_TYPE, "overheat");
}

static void thermal_action(struct thermal_info *info, void *data)
{
	if (!info)
		return;

	switch (info->state) {
	case THERMAL_STATE_LOW:
		_E("THERMAL: temparature LOW");
		thermal_remove_noti();
		break;
	case THERMAL_STATE_NORMAL:
		_I("THERMAL: temparature normal");
		thermal_remove_noti();
		break;
	case THERMAL_STATE_HIGH:
		if (info->level == THERMAL_LEVEL_WARNING) {
			_E("THERMAL: temparature HIGH (warning)");
			thermal_add_noti();
		} else if (info->level == THERMAL_LEVEL_CRITICAL) {
			_E("THERMAL: temparature HIGH (critical)");
			thermal_remove_noti();
			thermal_add_popup();
		} else if (info->level == THERMAL_LEVEL_POWEROFF) {
			_E("THERMAL: temparature HIGH (poweroff)");
			thermal_remove_noti();
			thermal_add_popup();
		} else
			_E("THERMAL: temparature HIGH");
		break;
	default:
		break;
	}
}

static int thermal_service_start(void)
{
	int ret;

	if (!thermal_dev) {
		_E("Thermal service is not supported");
		return -ENOTSUP;
	}

	ret = thermal_dev->register_changed_event(thermal_action, NULL);
	if (ret < 0)
		_E("Failed to register thermal event (%d)", ret);

	return ret;
}

static int thermal_service_stop(void)
{
	int ret;

	if (!thermal_dev) {
		_E("Thermal service is not supported");
		return -ENOTSUP;
	}

	ret = thermal_dev->unregister_changed_event(thermal_action);
	if (ret < 0)
		_E("Failed to unregister thermal event (%d)", ret);

	return ret;
}

static int booting_done(void *data)
{
	static int done;

	if (data == NULL)
		return done;
	done = *(int *)data;
	if (done == 0)
		return done;

	_I("booting done");
	thermal_service_start();

	return done;
}

static void thermal_init(void *data)
{
	register_notifier(DEVICE_NOTIFIER_BOOTING_DONE, booting_done);
}

static void thermal_exit(void *data)
{
	unregister_notifier(DEVICE_NOTIFIER_BOOTING_DONE, booting_done);
	thermal_service_stop();
}

static int thermal_probe(void *data)
{
	struct hw_info *info;
	int ret;

	if (thermal_dev)
		return 0;

	ret = hw_get_info(THERMAL_HARDWARE_DEVICE_ID,
			(const struct hw_info **)&info);
	if (ret < 0) {
		_E("There is no HAL for Thermal");
		return ret;
	}

	if (!info->open) {
		_E("Failed to open thermal device; open(NULL)");
		return -ENODEV;
	}

	ret = info->open(info, NULL, (struct hw_common**)&thermal_dev);
	if (ret < 0) {
		_E("Failed to get thermal device structure (%d)", ret);
		return ret;
	}

	if (!thermal_dev ||
		!thermal_dev->get_state ||
		!thermal_dev->register_changed_event ||
		!thermal_dev->unregister_changed_event) {
		_E("Thermal HAL does not support event handler");
		return -ENODEV;
	}

	_I("Thermal device structure load success");
	return 0;
}

static const struct device_ops thermal_ops = {
	.name     = "thermal",
	.probe    = thermal_probe,
	.init     = thermal_init,
	.exit     = thermal_exit,
};

DEVICE_OPS_REGISTER(&thermal_ops)
